# coding:utf-8
from typing import Dict

from pysubway.utils.ustring import random_key

from .base import CloudWalkOCRBase


class IdCard(CloudWalkOCRBase):
    not_get_face = '0'
    get_face = '1'
    specified = ''
    uri = '/ai-cloud-face/ocr/idcard'

    @property
    def cls_name(self) -> str:
        return IdCard.__name__

    def generate_send_data(self, img: str) -> Dict[str, str]:
        send_data = {
            "img": img,
            "appKey": self.app_key,
            "nonceStr": self.specified or random_key(16),
            "getFace": self.get_face,
        }
        sign = self.generate_sign(send_data)
        send_data['sign'] = sign
        return send_data



import json
from typing import Dict, Any, Optional, List, Union, Tuple

import pandas as pd
import requests
from pysubway.utils import udict
from pysubway.utils.encryption.aes import AESCrypt
from pysubway.utils.encryption.rsa2 import RSAEncryption


class BoolDataBase:
    """
    BoolDataBase params frame:
    sign
    biz_data
    institution_id

    BoolDataBase result frame:
    {
        'resp_code':'',
        'resp_data': {},
        "resp_msg": "认证成功",
        "resp_order": "018011562838612454492953",
        "timestamp": "20190711175017"
    }
    """
    urls = {
        'local': 'http://localhost:8000/api/lightning/product/query',
        'sit': 'http://guard.shouxin168.net/api/lightning/product/query',
        'prod': 'https://guard.shouxin168.com/api/lightning/product/query'
    }
    resp_code_biz_data_error = ('SW0017',
                                'SW0018',
                                'SW0019',
                                'SW0020',
                                'SW1009',
                                'SW1010',
                                'SW1011',
                                'SW1012'
                                )
    df_default_header: Tuple[str, ...] = tuple()

    def __init__(self, rsa_prv_key: str, aes_key: str, company_uuid: str, url: str = ''):
        """

        :param rsa_prv_key:
        :param aes_key:
        :param company_uuid:
        :param url:

        self.resp_data is BoolDataBase resp result

        """
        self.url: str = url or self.urls['prod']
        self.rsa_prv_key: str = rsa_prv_key
        self.aes_key: str = aes_key
        self.company_uuid: str = company_uuid
        self.response: Dict = dict()
        self.biz_data: Dict = dict()

    @staticmethod
    def generate_biz_data(*args: str) -> Dict[str, Any]:
        raise NotImplementedError()

    @property
    def default_resp_data(self) -> Dict[str, Any]:
        raise NotImplementedError()

    def set_biz_data(self, *args: str) -> 'BoolDataBase':
        self.biz_data = self.generate_biz_data(*args)
        return self

    def encrypt_biz_data(self, biz_data: Dict[str, str]) -> str:
        sorted_str = json.dumps(biz_data, sort_keys=True)
        return AESCrypt(self.aes_key).encrypt(sorted_str)

    def generate_sign(self, biz_data: Dict[str, str]) -> str:
        sorted_str = udict.Dict(biz_data).sort_keys()
        print('sorted_str ++++++++++', sorted_str)
        return RSAEncryption.generate_signature(sorted_str, self.rsa_prv_key)

    def request(self, biz_data: Optional[Dict[str, Any]] = None) -> Dict[str, Any]:
        biz_data = biz_data or self.biz_data
        data = {
            'sign': self.generate_sign(biz_data),
            'biz_data': self.encrypt_biz_data(biz_data),
            'institution_id': self.company_uuid,
        }
        resp = requests.post(self.url, data=data)
        if resp.ok:
            self.response = resp.json()
            return self.response
        else:
            raise ValueError(resp.text)

    @staticmethod
    def is_success_request(resp: Dict[str, Any]) -> bool:
        if isinstance(resp, dict) and resp.get('resp_code') == 'SW0000':
            return True
        else:
            print(resp)
            return False

    def is_succeed(self, resp: Dict[str, Any] = None) -> bool:
        """
        the function is same to is_success_request,
        but support to chain operate
        :param resp:
        :return:
        """
        response = self.response if resp is None else resp
        return self.is_success_request(response)

    def result_as_df(self, valuable_part: Union[List, Dict, None],
                     default_part: Union[List, Dict, None],
                     append_biz_data: bool = True) -> pd.DataFrame:
        """
        transfer booldata result(type dict) to pandas.dataframe
        this method often is used to generate excel.

        :param valuable_part: valuable part of booldata.resp_data
            example: self.response.get('resp_data', {}).get('data_list')
        :param default_part: return default part when valuable_part is empty,
            default value refer to person_law.
        :param append_biz_data: append biz_data to return value
        if data_list is empty, return default_data_list
        if append_biz_data, append biz_data to return value
        :return:
        """
        # valuable = valuable_part or self.response.get('resp_data', {})
        # interface personal_law valuable_part is {'data_list': [], 'total': 0}
        valuable = valuable_part
        valuable = valuable or default_part
        if append_biz_data:
            biz_data = udict.Dict(self.biz_data).remove(('service', 'mode'))
            if isinstance(valuable, list):
                new_valuable = []
                for val_data in valuable:
                    val_data.update(biz_data)
                    new_valuable.append(val_data)
                valuable = new_valuable
            elif isinstance(valuable, dict):
                valuable.update(biz_data)
        try:
            return pd.DataFrame(valuable)
        except ValueError as e:
            return pd.DataFrame(valuable, index=[0])

    # ValueError: If using all scalar values, you must pass an index

    def result_as_df_partial(self, append_biz_data: bool = True) -> pd.DataFrame:
        """
        subclass wrap method result_as_df partially
        :param append_biz_data:
        :return:
        """
        raise NotImplementedError()



from typing import Dict

from pysubway.utils import udict

from .base import SuixingfuBase


class BankcardTripleElements(SuixingfuBase):
    verifyType = '03'

    def generate_biz_data(self, name: str, identNo: str, cardNo: str) -> Dict[str, str]:
        send_data = udict.Dict.filter(locals())
        send_data['verifyType'] = self.verifyType
        return send_data



import hashlib
from typing import Dict, Any
from urllib.parse import urlencode
from urllib.parse import urljoin

import requests
from pysubway.errors import IncomingDataError
from pysubway.utils import udict
from pysubway.utils import url
from pysubway.utils.hash import hash_hmac
from pysubway.utils.utime import timestamp


class BangShengBase:
    u"""
    邦盛接口管理, 它有请求邦盛接口和解析邦盛接口返回值的能力
    邦盛接口有 ip白名单 限制，本地调不通的;
    接口传入的时间与标准时间差15分钟以上，抛出异常：{"code":1016,"message":"Unauthorized","status":401}
    参数异常，{"code":1010,"message":"parameters verify error","status":400}
    之前的接口, 业务码都是在 json 中判断；
    但是邦盛的接口，如果参数错误则直接 400 了。
    在邦盛的测试环境，参数错误和时间限制，这两个异常都测试不出来;
    """

    def __init__(self, host: str, uri: str, app_key: str, app_secret: str):
        self.host: str = host
        self.uri: str = uri
        self.url: str = urljoin(self.host, self.uri)
        self.app_key: str = app_key
        self.app_secret: str = app_secret
        self.date: str = timestamp(precision='ms', returned_str=True)
        self.full_url: str = ''
        self.header: Dict = dict()
        self.biz_data: Dict[str, str] = dict()

    def generate_full_url(self, biz_data: Dict[str, Any]) -> str:
        u"""
        request 请求的 url, 映射 demo 中的 httpget
        原始逻辑：url + urlPath + "?" + encodeParam
        url 接口地址 + urlPath api 地址 + '?' + 转码后的参数
        :return: str
        """
        return '?'.join((self.url, urlencode(biz_data)))

    def set_full_url(self, biz_data: Dict[str, Any] = None) -> 'BangShengBase':
        biz_data = biz_data or self.biz_data
        self.full_url = self.generate_full_url(biz_data)
        return self

    def generate_sign(self, biz_data: Dict[str, Any]) -> str:
        u"""
        请求中 header 头部的 X-BS-Validate 对应的值
        代码逻辑：
        String validate = HmacUtils.hmacMd5Hex(
            APP_SECRET.getBytes(), signString.getBytes(Charset.forName("UTF-8")));
        :return: str
        """
        joined_uri = url.urlencode(biz_data, safe=False)
        raw_sign = '\n'.join((self.uri, joined_uri, self.date))
        return hash_hmac(self.app_secret, raw_sign, digestmod=hashlib.md5)

    def generate_header(self, biz_data: Dict[str, Any]) -> Dict[str, Any]:
        return {
            'X-BS-App-Key': self.app_key,
            'X-BS-Date': self.date,
            'X-BS-Validate': self.generate_sign(biz_data),
        }

    def set_header(self, biz_data: Dict[str, Any] = None) -> 'BangShengBase':
        biz_data = biz_data or self.biz_data
        self.header = self.generate_header(biz_data)
        return self

    @staticmethod
    def generate_biz_data(name: str = '', mobile: str = '', idCard: str = '') -> Dict[str, str]:
        return udict.Dict.filter(locals())

    def set_biz_data(self, name: str, mobile: str, idCard: str) -> 'BangShengBase':
        self.biz_data = self.generate_biz_data(name=name, mobile=mobile, idCard=idCard)
        return self

    def request(self, full_url: str = '', headers: Dict[str, str] = None) -> Dict[str, str]:
        u"""
        请求邦盛的接口
        encourage chain operation:
        BangShengBase().set_biz_data().set_full_url().set_header().request()
        """
        full_url = full_url or self.full_url
        headers = headers or self.header
        response = requests.get(full_url, headers=headers)
        if response.ok:
            return response.json()
        elif response.status_code == requests.codes['bad_request']:
            code = response.json().get('code')
            if code == 1010:
                raise IncomingDataError(f'text {response.text} code {code} is unexpected')
            else:
                raise ValueError(f'text {response.text} code {code} is unexpected')
        else:
            raise ValueError(f'text {response.text} and status_code {response.status_code} is unexpected')



from typing import Dict, Optional, Tuple, Any

import pandas as pd
from pysubway.errors import RequestFailed
from pysubway.utils import udict
from pysubway.utils.file import File
from pysubway.utils.uexcel import Excel

from .base import BoolDataBase


class Network3Eles(BoolDataBase):
    service = 'network_operator_triple_elements'
    mode = 'mode_triple_elements'
    default_part = {
        'name': '',
        'phone': '',
        'idcard': ''
    }
    df_default_header = (
        "name",
        "phone",
        "ident_number",
        "result_code",
        "result_code_desc",
    )

    @classmethod
    def generate_biz_data(cls, name: str, phone: str, ident_number: str) -> Dict[str, Any]:
        biz_data = udict.Dict.filter(locals())
        biz_data.update({
            "service": cls.service,
            "mode": cls.mode
        })
        return biz_data

    def set_biz_data(self, name: str, phone: str, ident_number: str) -> 'Network3Eles':
        self.biz_data = self.generate_biz_data(name, phone, ident_number)
        return self

    def result_as_df_partial(self, append_biz_data: bool = True) -> pd.DataFrame:
        return self.result_as_df(self.response.get('resp_data', {}), self.default_part,
                                 append_biz_data=append_biz_data)


def batch_test(input: str,
               account: Dict[str, str],
               output_file: str,
               names: Optional[Tuple[str, ...]] = ('name', 'idcard', 'phone'),
               columns: Tuple[str, ...] = Network3Eles.df_default_header,
               raise_failed_exception: bool = False,
               error_log: Optional[str] = None,
               ) -> None:
    """
    batch test personal law, read from excel -> request personal law -> save result to excel

    :param input: excel file
    :param account: booldata account
    :param output_file: xlsx file that saving booldata personal law result
    :param names: input excel file header, you can adjust the sequence
    :param columns: output_file columns header
    :return: None
    """
    error_log = error_log or File.join_path(File(input).dirname, '.'.join((File(input).pure_name, 'error_log')))
    excel_ins = Excel(input).read(names=names)
    result_df = pd.DataFrame()
    guard_ins = Network3Eles(account['rsa_prv_key'], account['aes_key'], account['company_uuid'])
    for _, series in excel_ins.dataframe.iterrows():
        name = series['name']
        phone = series['phone']
        idcard = series['idcard']
        guard_result = guard_ins.set_biz_data(name, phone, idcard).request()
        if not guard_ins.is_succeed():
            if raise_failed_exception:
                raise RequestFailed(f'{guard_result}')
            print('error ins_result  ----> ', guard_result)
            with open(error_log, mode='a') as f:
                print(f"param is {series['idcard']} {series['name']} {series['phone']}")
                f.write(f"{series['idcard']},{series['name']},{series['phone']}\n")
        guard_result = guard_ins.result_as_df_partial()
        result_df = result_df.append(guard_result)
        print(f'{name} {phone} {idcard} result_df', result_df)
        # break
    result_df.to_excel(output_file, columns=columns, index=False)


if __name__ == '__main__':
    from gitignore import BOOLDATA_ACCOUNT

    # 姓名电话身份证银行卡
    file = '/Users/yutou/shouxin/sxProject/pysubway/pysubway/gitignore/chebei.xlsx'
    output = '/Users/yutou/shouxin/sxProject/pysubway/pysubway/gitignore/chebei_network3eles_result.xlsx'
    # with idcard
    batch_test(file, BOOLDATA_ACCOUNT, output, names=('name', 'phone', 'idcard', 'bankcard'))
